from django.forms.models import inlineformset_factory

from .widgets import FormWidget, FormSetWidget


class BaseCompositeField(object):
    """
    The ``BaseCompositeField`` takes care of keeping some kind of compatibility
    with the ``django.forms.Field`` class.
    """

    widget = None
    show_hidden_initial = False

    # Tracks each time a FormSetField instance is created. Used to retain
    # order.
    creation_counter = 0

    def __init__(self, required=True, widget=None, label=None, help_text='',
                 localize=False):
        self.required = required
        self.label = label
        self.help_text = help_text

        widget = widget or self.widget
        if isinstance(widget, type):
            widget = widget()

        # Trigger the localization machinery if needed.
        self.localize = localize
        if self.localize:
            widget.is_localized = True

        # Let the widget know whether it should display as required.
        widget.is_required = self.required

        # We do not call self.widget_attrs() here as the original field is
        # doing it.

        self.widget = widget

        # Increase the creation counter, and save our local copy.
        self.creation_counter = BaseCompositeField.creation_counter
        BaseCompositeField.creation_counter += 1


class CompositeField(BaseCompositeField):
    """
    Implements the base structure that is relevant for all composite fields.
    This field cannot be used directly, use a subclass of it.
    """

    prefix_name = 'composite'

    def __init__(self, *args, **kwargs):
        super(CompositeField, self).__init__(*args, **kwargs)

        # Let the widget know about the field for easier complex renderings in
        # the template.
        self.widget.field = self

    def get_prefix(self, form, name):
        """
        Return the prefix that is used for the formset.
        """
        return '{form_prefix}{prefix_name}-{field_name}'.format(
            form_prefix=form.prefix + '-' if form.prefix else '',
            prefix_name=self.prefix_name,
            field_name=name)

    def get_initial(self, form, name):
        """
        Get the initial data that got passed into the superform for this
        composite field. It should return ``None`` if no initial values where
        given.
        """

        if hasattr(form, 'initial'):
            return form.initial.get(name, None)
        return None

    def get_kwargs(self, form, name):
        """
        Return the keyword arguments that are used to instantiate the formset.
        """
        kwargs = {
            'prefix': self.get_prefix(form, name),
            'initial': self.get_initial(form, name),
        }
        kwargs.update(self.default_kwargs)
        return kwargs


class FormField(CompositeField):
    """
    A field that can be used to nest a form inside another form::

        from django import forms
        from django_superform import SuperForm

        class AddressForm(forms.Form):
            street = forms.CharField()
            city = forms.CharField()

        class RegistrationForm(SuperForm):
            first_name = forms.CharField()
            last_name = forms.CharField()
            address = FormField(AddressForm)

    You can then display the fields in the template with (given that
    ``registration_form`` is an instance of ``RegistrationForm``)::

        {{ registration_form.address.street }}
        {{ registration_form.address.street.errors }}
        {{ registration_form.address.city }}
        {{ registration_form.address.city.errors }}

    The fields will all have a prefix in their name so that the naming does not
    clash with other fields on the page. The name attribute of the input tag
    for the ``street`` field in this example will be: ``form-address-street``.
    The name will change if you set a prefix on the superform::

        form = RegistrationForm(prefix='registration')

    Then the field name will be ``registration-form-address-street``.

    You can pass the ``kwargs`` argument to the ``__init__`` method in order to
    give keyword arguments that you want to pass through to the form when it is
    instaniated. So you could use this to pass in initial values::

        class RegistrationForm(SuperForm):
            address = FormField(AddressForm, kwargs={
                'initial': {'street': 'Stairway to Heaven 1'}
            })

    But you can also use nested initial values which you pass into the
    superform::

        RegistrationForm(initial={
            'address': {'street': 'Highway to Hell 666'}
        })

    The first method (using ``kwargs``) will take precedence.
    """

    prefix_name = 'form'
    widget = FormWidget

    def __init__(self, form_class, kwargs=None, **field_kwargs):
        super(FormField, self).__init__(**field_kwargs)

        self.form_class = form_class
        if kwargs is None:
            kwargs = {}
        self.default_kwargs = kwargs

    def get_form_class(self, form, name):
        """
        Return the form class that will be used for instantiation in
        ``get_form``. You can override this method in subclasses to change
        the behaviour of the given form class.
        """
        return self.form_class

    def get_form(self, form, name):
        """
        Get an instance of the form.
        """
        kwargs = self.get_kwargs(form, name)
        form_class = self.get_form_class(form, name)
        composite_form = form_class(
            form.data if form.is_bound else None,
            form.files if form.is_bound else None,
            **kwargs)
        return composite_form


class ModelFormField(FormField):
    """
    This class is the to :class:`~django_superform.fields.FormField` what
    Django's :class:`ModelForm` is to :class:`Form`. It has the same behaviour
    as :class:`~django_superform.fields.FormField` but will also save the
    nested form if the super form is saved. Here is an example::

        from django_superform import ModelFormField

        class EmailForm(forms.ModelForm):
            class Meta:
                model = EmailAddress
                fields = ('email',)

        class UserForm(SuperModelForm):
            email = ModelFormField(EmailForm)

            class Meta:
                model = User
                fields = ('username',)

        user_form = UserForm(
            {'username': 'john', 'form-email-email': 'john@example.com'})
        if user_form.is_valid():
            user_form.save()

    This will save the ``user_form`` and create a new instance of ``User``
    model and it will also save the ``EmailForm`` and therefore create an
    instance of ``EmailAddress``!

    However you usually want to use one of the exsting subclasses, like
    :class:`~django_superform.fields.ForeignKeyFormField` or extend from
    ``ModelFormField`` class and override the
    :meth:`~django_superform.fields.ModelFormField.get_instance` method.

    .. note::
        Usually the :class:`~django_superform.fields.ModelFormField` is used
        inside a :class:`~django_superform.forms.SuperModelForm`. You actually
        can use it within a :class:`~django_superform.forms.SuperForm`, but
        since this form type does not have a ``save()`` method, you will need
        to take care of saving the nested model form yourself.
    """

    def get_instance(self, form, name):
        """
        Provide an instance that shall be used when instantiating the
        modelform. The ``form`` argument is the super-form instance that this
        ``ModelFormField`` is used in. ``name`` is the name of this field on
        the super-form.

        This returns ``None`` by default. So you usually want to override this
        method in a subclass.
        """
        return None

    def get_kwargs(self, form, name):
        """
        Return the keyword arguments that are used to instantiate the formset.

        The ``instance`` kwarg will be set to the value returned by
        :meth:`~django_superform.fields.ModelFormField.get_instance`. The
        ``empty_permitted`` kwarg will be set to the inverse of the
        ``required`` argument passed into the constructor of this field.
        """
        kwargs = super(ModelFormField, self).get_kwargs(form, name)
        instance = self.get_instance(form, name)
        kwargs.setdefault('instance', instance)
        kwargs.setdefault('empty_permitted', not self.required)
        return kwargs

    def shall_save(self, form, name, composite_form):
        """
        Return ``True`` if the given ``composite_form`` (the nested form of
        this field) shall be saved. Return ``False`` if the form shall not be
        saved together with the super-form.

        By default it will return ``False`` if the form was not changed and the
        ``empty_permitted`` argument for the form was set to ``True``. That way
        you can allow empty forms.
        """
        if composite_form.empty_permitted and not composite_form.has_changed():
            return False
        return True

    def save(self, form, name, composite_form, commit):
        """
        This method is called by
        :meth:`django_superform.forms.SuperModelForm.save` in order to save the
        modelform that this field takes care of and calls on the nested form's
        ``save()`` method. But only if
        :meth:`~django_superform.fields.ModelFormField.shall_save` returns
        ``True``.
        """
        if self.shall_save(form, name, composite_form):
            return composite_form.save(commit=commit)
        return None


class ForeignKeyFormField(ModelFormField):
    def __init__(self, form_class, kwargs=None, field_name=None, blank=None,
                 **field_kwargs):
        super(ForeignKeyFormField, self).__init__(form_class, kwargs,
                                                  **field_kwargs)
        self.field_name = field_name
        self.blank = blank

    def get_kwargs(self, form, name):
        kwargs = super(ForeignKeyFormField, self).get_kwargs(form, name)
        if 'instance' not in kwargs:
            kwargs.setdefault('instance', self.get_instance(form, name))
        if 'empty_permitted' not in kwargs:
            if self.allow_blank(form, name):
                kwargs['empty_permitted'] = True
        return kwargs

    def get_field_name(self, form, name):
        return self.field_name or name

    def allow_blank(self, form, name):
        """
        Allow blank determines if the form might be completely empty. If it's
        empty it will result in a None as the saved value for the ForeignKey.
        """
        if self.blank is not None:
            return self.blank
        model = form._meta.model
        field = model._meta.get_field(self.get_field_name(form, name))
        return field.blank

    def get_form_class(self, form, name):
        form_class = self.form_class
        return form_class

    def get_instance(self, form, name):
        field_name = self.get_field_name(form, name)
        return getattr(form.instance, field_name)

    def save(self, form, name, composite_form, commit):
        # Support the ``empty_permitted`` attribute. This is set if the field
        # is ``blank=True`` .
        if composite_form.empty_permitted and not composite_form.has_changed():
            saved_obj = composite_form.instance
        else:
            saved_obj = super(ForeignKeyFormField, self).save(form, name,
                                                              composite_form,
                                                              commit)
        setattr(form.instance, self.get_field_name(form, name), saved_obj)
        if commit:
            form.instance.save()
        else:
            raise NotImplementedError(
                'ForeignKeyFormField cannot yet be used with non-commiting '
                'form saves.')
        return saved_obj


class FormSetField(CompositeField):
    """
    First argument is a formset class that is instantiated by this
    FormSetField.

    You can pass the ``kwargs`` argument to specify kwargs values that
    are used when the ``formset_class`` is instantiated.
    """

    prefix_name = 'formset'
    widget = FormSetWidget

    def __init__(self, formset_class, kwargs=None, **field_kwargs):
        super(FormSetField, self).__init__(**field_kwargs)

        self.formset_class = formset_class
        if kwargs is None:
            kwargs = {}
        self.default_kwargs = kwargs

    def get_formset_class(self, form, name):
        """
        Return the formset class that will be used for instantiation in
        ``get_formset``. You can override this method in subclasses to change
        the behaviour of the given formset class.
        """
        return self.formset_class

    def get_formset(self, form, name):
        """
        Get an instance of the formset.
        """
        kwargs = self.get_kwargs(form, name)
        formset_class = self.get_formset_class(form, name)
        formset = formset_class(
            form.data if form.is_bound else None,
            form.files if form.is_bound else None,
            **kwargs)
        return formset


class ModelFormSetField(FormSetField):
    def shall_save(self, form, name, formset):
        return True

    def save(self, form, name, formset, commit):
        if self.shall_save(form, name, formset):
            return formset.save(commit=commit)
        return None


class InlineFormSetField(ModelFormSetField):
    """
    The ``InlineFormSetField`` helps when you want to use a inline formset.

    You can pass in either the keyword argument ``formset_class`` which is a
    ready to use formset that inherits from ``BaseInlineFormSet`` or was
    created by the ``inlineformset_factory``.

    The other option is to provide the arguments that you would usually pass
    into the ``inlineformset_factory``. The required arguments for that are:

    ``model``
        The model class which should be represented by the forms in the
        formset.
    ``parent_model``
        The parent model is the one that is referenced by the model in a
        foreignkey.
    ``form`` (optional)
        The model form that is used as a baseclass for the forms in the inline
        formset.

    You can use the ``kwargs`` keyword argument to pass extra arguments for the
    formset that are passed through when the formset is instantiated.

    All other not mentioned keyword arguments, like ``extra``, ``max_num`` etc.
    will be passed directly to the ``inlineformset_factory``.

    Example:

        class Gallery(models.Model):
            name = models.CharField(max_length=50)

        class Image(models.Model):
            gallery = models.ForeignKey(Gallery)
            image = models.ImageField(...)

        class GalleryForm(ModelFormWithFormSets):
            class Meta:
                model = Gallery
                fields = ('name',)

            images = InlineFormSetField(
                parent_model=Gallery,
                model=Image,
                extra=1)
    """

    def __init__(self, parent_model=None, model=None, formset_class=None,
                 kwargs=None, **factory_kwargs):
        """
        You need to either provide the ``formset_class`` or the ``model``
        argument.

        If the ``formset_class`` argument is not given, the ``model`` argument
        is used to create the formset_class on the fly when needed by using the
        ``inlineformset_factory``.
        """

        # Make sure that all standard arguments will get passed through to the
        # parent's __init__ method.
        field_kwargs = {}
        for arg in ['required', 'widget', 'label', 'help_text', 'localize']:
            if arg in factory_kwargs:
                field_kwargs[arg] = factory_kwargs.pop(arg)

        self.parent_model = parent_model
        self.model = model
        self.formset_factory_kwargs = factory_kwargs
        super(InlineFormSetField, self).__init__(formset_class, kwargs=kwargs,
                                                 **field_kwargs)
        if (
                self.formset_class is None and
                'fields' not in self.formset_factory_kwargs and
                'exclude' not in self.formset_factory_kwargs):
            raise ValueError(
                'You need to either specify the `formset_class` argument or '
                'the `fields`/`exclude` arguments when creating a {0}.'
                .format(self.__class__.__name__))

    def get_model(self, form, name):
        return self.model

    def get_parent_model(self, form, name):
        if self.parent_model is not None:
            return self.parent_model
        return form._meta.model

    def get_formset_class(self, form, name):
        """
        Either return the formset class that was provided as argument to the
        __init__ method, or build one based on the ``parent_model`` and
        ``model`` attributes.
        """
        if self.formset_class is not None:
            return self.formset_class
        formset_class = inlineformset_factory(
            self.get_parent_model(form, name),
            self.get_model(form, name),
            **self.formset_factory_kwargs)
        return formset_class

    def get_kwargs(self, form, name):
        kwargs = super(InlineFormSetField, self).get_kwargs(form, name)
        kwargs.setdefault('instance', form.instance)
        return kwargs



from django import forms
from django.template import loader


class TemplateWidget(forms.Widget):
    """
    Template based widget. It renders the ``template_name`` set as attribute
    which can be overriden by the ``template_name`` argument to the
    ``__init__`` method.
    """

    field = None
    template_name = None
    value_context_name = None

    def __init__(self, *args, **kwargs):
        template_name = kwargs.pop('template_name', None)
        if template_name is not None:
            self.template_name = template_name
        super(TemplateWidget, self).__init__(*args, **kwargs)
        self.context_instance = None

    def get_context_data(self):
        return {}

    def get_context(self, name, value, attrs=None):
        context = {
            'name': name,
            'hidden': self.is_hidden,
            'required': self.is_required,
            'True': True,
            # In our case ``value`` is the form or formset instance.
            'value': value,
        }
        if self.value_context_name:
            context[self.value_context_name] = value

        # True is injected in the context to allow stricter comparisons
        # for widget attrs. See #25.
        if self.is_hidden:
            context['hidden'] = True

        context.update(self.get_context_data())
        context['attrs'] = self.build_attrs(attrs)

        for key, attr in context['attrs'].items():
            if attr == 1:
                # 1 == True so 'key="1"' will show up only as 'key'
                # Casting to a string so that it doesn't equal to True
                # See #25.
                if not isinstance(attr, bool):
                    context['attrs'][key] = str(attr)

        return context

    def render(self, name, value, attrs=None, **kwargs):
        template_name = kwargs.pop('template_name', None)
        if template_name is None:
            template_name = self.template_name
        context = self.get_context(name, value, attrs=attrs or {}, **kwargs)
        return loader.render_to_string(
            template_name,
            dictionary=context,
            context_instance=self.context_instance)


class FormWidget(TemplateWidget):
    template_name = 'superform/formfield.html'
    value_context_name = 'form'


class FormSetWidget(TemplateWidget):
    template_name = 'superform/formsetfield.html'
    value_context_name = 'formset'



from django import forms
from django.test import TestCase
from django_superform import SuperForm, FormField


class AddressForm(forms.Form):
    street = forms.CharField()
    city = forms.CharField()

    def __init__(self, *args, **kwargs):
        self.custom_attribute = kwargs.pop('custom_kwarg', None)
        super(AddressForm, self).__init__(*args, **kwargs)


class MultiAddressForm(SuperForm):
    address1 = FormField(AddressForm)
    address2 = FormField(AddressForm)


class RegistrationForm(SuperForm):
    first_name = forms.CharField()
    last_name = forms.CharField()
    address = FormField(AddressForm)
    address_custom_kwarg = FormField(AddressForm, kwargs={'custom_kwarg': True})
    address_initial = FormField(AddressForm, kwargs={
        'initial': {
            'street': 'Homebase 42',
            'city': 'Supertown'
        }
    })
    more_addresses = FormField(MultiAddressForm)


class FormFieldTests(TestCase):
    def test_is_in_composite_fields(self):
        superform = RegistrationForm()
        self.assertTrue('address' in superform.composite_fields)

    def test_is_in_forms_attr(self):
        superform = RegistrationForm()
        self.assertTrue('address' in superform.forms)
        self.assertTrue(isinstance(superform.forms['address'], AddressForm))

    def test_form_prefix(self):
        superform = RegistrationForm()
        form = superform.forms['address']
        self.assertEqual(form.prefix, 'form-address')

    def test_field_name(self):
        superform = RegistrationForm()
        boundfield = superform.forms['address']['street']
        self.assertEqual(boundfield.html_name, 'form-address-street')

        superform = RegistrationForm(prefix='registration')
        boundfield = superform.forms['address']['street']
        self.assertEqual(boundfield.html_name, 'registration-form-address-street')

    def test_form_kwargs(self):
        superform = RegistrationForm()
        form = superform.forms['address_custom_kwarg']
        self.assertEqual(form.custom_attribute, True)

    def test_form_kwargs_initial(self):
        superform = RegistrationForm()
        form = superform.forms['address_initial']
        self.assertEqual(form['street'].value(), 'Homebase 42')
        self.assertEqual(form['city'].value(), 'Supertown')

    def test_initial_pass_through(self):
        superform = RegistrationForm(initial={
            'first_name': 'Patricia',
            'address': {'street': 'Default Road'}
        })
        form = superform.forms['address']
        self.assertEqual(superform['first_name'].value(), 'Patricia')
        self.assertEqual(superform['last_name'].value(), None)
        self.assertEqual(form['street'].value(), 'Default Road')
        self.assertEqual(form['city'].value(), None)

    def test_initial_pass_through_with_multiple_layers(self):
        superform = RegistrationForm(initial={
            'more_addresses': {
                'address1': {
                    'street': 'Fooway',
                    'city': 'Testcity'
                },
                'address2': {
                    'street': 'Barboulevard',
                }
            }
        })
        address1 = superform.forms['more_addresses'].forms['address1']
        address2 = superform.forms['more_addresses'].forms['address2']
        self.assertEqual(address1['street'].value(), 'Fooway')
        self.assertEqual(address1['city'].value(), 'Testcity')
        self.assertEqual(address2['street'].value(), 'Barboulevard')
        self.assertEqual(address2['city'].value(), None)



from django import forms
from django.forms.forms import ErrorList
from django.forms.formsets import formset_factory
from django.test import TestCase
from django_superform import SuperForm, FormSetField


class EmailForm(forms.Form):
    email = forms.EmailField()


EmailFormSet = formset_factory(EmailForm)


class AccountForm(SuperForm):
    username = forms.CharField()
    emails = FormSetField(EmailFormSet)


class SuperFormTests(TestCase):
    def test_declared_composite_fields(self):
        self.assertEqual(list(AccountForm.base_fields.keys()), ['username'])
        self.assertTrue(hasattr(AccountForm, 'composite_fields'))
        self.assertEqual(list(AccountForm.composite_fields.keys()), ['emails'])
        field = AccountForm.composite_fields['emails']
        self.assertIsInstance(field, FormSetField)

        self.assertFalse(hasattr(AccountForm, 'formsets'))

    def test_fields_in_instantiated_forms(self):
        form = AccountForm()

        self.assertTrue(hasattr(form, 'composite_fields'))
        self.assertTrue(hasattr(form, 'formsets'))

        self.assertEqual(list(form.formsets.keys()), ['emails'])
        formset = form.formsets['emails']
        self.assertIsInstance(formset, EmailFormSet)


class FormSetsInSuperFormsTests(TestCase):
    def setUp(self):
        self.formset_data = {
            'formset-emails-INITIAL_FORMS': 0,
            'formset-emails-TOTAL_FORMS': 1,
            'formset-emails-MAX_NUM_FORMS': 3,
        }

    def test_prefix(self):
        form = AccountForm()
        formset = form.formsets['emails']
        self.assertEqual(formset.prefix, 'formset-emails')

    def test_validation(self):
        data = self.formset_data.copy()
        form = AccountForm(data)
        self.assertEqual(form.is_valid(), False)
        self.assertTrue(form.errors['username'])
        self.assertFalse('emails' in form.errors)

        data = self.formset_data.copy()
        data['formset-emails-INITIAL_FORMS'] = 1
        form = AccountForm(data)
        self.assertEqual(form.is_valid(), False)
        self.assertTrue(form.errors['username'])
        self.assertTrue(form.errors['emails'])
        self.assertIsInstance(form.errors['emails'], ErrorList)



"""
This is awesome. And needs more documentation.

To bring some light in the big number of classes in this file:

First there are:

* ``SuperForm``
* ``SuperModelForm``

They are the forms that you probably want to use in your own code. They are
direct base classes of ``django.forms.Form`` and ``django.forms.ModelForm``
and have the formset functionallity of this module backed in. They are ready
to use. Subclass them and be happy.

Then there are:

* ``SuperFormMixin``
* ``SuperModelFormMixin``

These are the mixins you can use if you don't want to subclass from
``django.forms.Form`` for whatever reason. The ones with Base at the beginning
don't have a metaclass attached. The ones without the Base in the name have
the relevant metaclass in place that handles the search for
``FormSetField``s.


Here is an example on how you can use this module::

    from django import forms
    from django_superform import SuperModelForm, FormSetField
    from .forms import CommentFormSet


    class PostForm(SuperModelForm):
        title = forms.CharField()
        text = forms.CharField()
        comments = FormSetField(CommentFormSet)

    # Now you can use the form in the view:

    def post_form(request):
        if request.method == 'POST':
            form = PostForm(request.POST, request.FILES)
            if form.is_valid():
                obj = form.save()
                return HttpResponseRedirect(obj.get_absolute_url())
        else:
            form = PostForm()
        return render_to_response('post_form.html', {
            'form',
        }, context_instance=RequestContext(request))

And yes, thanks for asking, the ``form.is_valid()`` and ``form.save()`` calls
transparantly propagate to the defined comments formset and call their
``is_valid()`` and ``save()`` methods. So you don't have to do anything
special in your view!

Now to how you can access the instantiated formsets::

    >>> form = PostForm()
    >>> form.composite_fields['comments']
    <CommetFormSet: ...>

Or in the template::

    {{ form.as_p }}

    {{ form.composite_fields.comments.management_form }}
    {% for fieldset_form in form.composite_fields.comments %}
        {{ fieldset_form.as_p }}
    {% endfor %}

You're welcome.

"""

from django import forms
from django.forms.forms import DeclarativeFieldsMetaclass, ErrorDict, ErrorList
from django.forms.models import ModelFormMetaclass
from django.utils import six
from .boundfield import CompositeBoundField
from .fields import CompositeField

try:
    from collections import OrderedDict
except ImportError:
    from django.utils.datastructures import SortedDict as OrderedDict


def get_declared_composite_fields(bases, attrs):
    """
    Create a list of CompositeField instances from the passed in `attrs`, plus
    any other composite fields of the base classes (in `bases`).
    """
    composite_fields = [
        (field_name, attrs.pop(field_name))
        for field_name, obj in list(six.iteritems(attrs))
        if isinstance(obj, CompositeField)]

    composite_fields.sort(key=lambda x: x[1].creation_counter)

    # If this class is subclassing another Form, add that Form's
    # composite_fields.
    # Note that we loop over the bases in *reverse*. This is necessary in
    # order to preserve the correct order of composite fields.
    for base in bases[::-1]:
        if hasattr(base, 'composite_fields'):
            composite_fields = (
                list(six.iteritems(base.composite_fields)) +
                composite_fields)

    return OrderedDict(composite_fields)


class DeclerativeCompositeFieldsMetaclass(type):
    """
    Metaclass that converts FormField and FormSetField attributes to a
    dictionary called `composite_fields`. It will also include all composite
    fields from parent classes.
    """

    def __new__(cls, name, bases, attrs):
        attrs['composite_fields'] = get_declared_composite_fields(bases, attrs)
        new_class = super(DeclerativeCompositeFieldsMetaclass, cls).__new__(
            cls, name, bases, attrs)
        return new_class


class SuperFormMetaclass(
        DeclerativeCompositeFieldsMetaclass,
        DeclarativeFieldsMetaclass):
    """
    Metaclass for :class:`~django_superform.forms.SuperForm`.
    """


class SuperModelFormMetaclass(
        DeclerativeCompositeFieldsMetaclass,
        ModelFormMetaclass):
    """
    Metaclass for :class:`~django_superform.forms.SuperModelForm`.
    """


class SuperFormMixin(object):
    """
    The base class for all super forms. It behaves just like a normal django
    form but will also take composite fields, like
    :class:`~django_superform.fields.FormField` and
    :class:`~django_superform.fields.FormSetField`.

    Cleaning, validation, etc. should work totally transparent.
    """

    def __init__(self, *args, **kwargs):
        super(SuperFormMixin, self).__init__(*args, **kwargs)
        self._init_composite_fields()

    def __getitem__(self, name):
        """
        Returns a BoundField for the given field name. It also returns
        CompositeBoundField instances for composite fields.
        """
        if name not in self.fields and name in self.composite_fields:
            field = self.composite_fields[name]
            return CompositeBoundField(self, field, name)
        return super(SuperFormMixin, self).__getitem__(name)

    def add_composite_field(self, name, field):
        """
        Add a dynamic composite field to the already existing ones and
        initialize it appropriatly.
        """
        self.composite_fields[name] = field
        self._init_composite_field(name, field)

    def get_composite_field_value(self, name):
        """
        Return the form/formset instance for the given field name.
        """
        field = self.composite_fields[name]
        if hasattr(field, 'get_form'):
            return self.forms[name]
        if hasattr(field, 'get_formset'):
            return self.formsets[name]

    def _init_composite_field(self, name, field):
        if hasattr(field, 'get_form'):
            form = field.get_form(self, name)
            self.forms[name] = form
        if hasattr(field, 'get_formset'):
            formset = field.get_formset(self, name)
            self.formsets[name] = formset

    def _init_composite_fields(self):
        """
        Setup the forms and formsets.
        """
        self.forms = OrderedDict()
        self.formsets = OrderedDict()
        for name, field in self.composite_fields.items():
            self._init_composite_field(name, field)

    def full_clean(self):
        """
        Clean the form, including all formsets and add formset errors to the
        errors dict.
        """
        super(SuperFormMixin, self).full_clean()
        for field_name, composite in self.forms.items():
            composite.full_clean()
            if not composite.is_valid():
                self._errors[field_name] = ErrorDict(composite.errors)
        for field_name, composite in self.formsets.items():
            composite.full_clean()
            if not composite.is_valid():
                self._errors[field_name] = ErrorList(composite.errors)


class SuperModelFormMixin(SuperFormMixin):
    def save(self, commit=True):
        """
        If ``commit=False`` django's modelform implementation will attach a
        ``save_m2m`` method to the form instance, so that you can call it
        manually later. When you call ``save_m2m``, the ``save_formsets``
        method will be executed as well.
        """
        saved_obj = super(SuperModelFormMixin, self).save(commit=commit)
        self.save_forms(commit=commit)
        self.save_formsets(commit=commit)
        return saved_obj

    def _extend_save_m2m(self, name, composites):
        additional_save_m2m = []
        for composite in composites:
            if hasattr(composite, 'save_m2m'):
                additional_save_m2m.append(composite.save_m2m)

        if not additional_save_m2m:
            return

        def additional_saves():
            for save_m2m in additional_save_m2m:
                save_m2m()

        # The save() method was called before save_forms()/save_formsets(), so
        # we will already have save_m2m() available.
        if hasattr(self, 'save_m2m'):
            _original_save_m2m = self.save_m2m
        else:
            _original_save_m2m = lambda: None

        def augmented_save_m2m():
            _original_save_m2m()
            additional_saves()

        self.save_m2m = augmented_save_m2m
        setattr(self, name, additional_saves)

    def save_forms(self, commit=True):
        saved_composites = []
        for name, composite in self.forms.items():
            field = self.composite_fields[name]
            if hasattr(field, 'save'):
                field.save(self, name, composite, commit=commit)
                saved_composites.append(composite)

        self._extend_save_m2m('save_forms_m2m', saved_composites)

    def save_formsets(self, commit=True):
        """
        Save all formsets. If ``commit=False``, it will modify the form's
        ``save_m2m()`` so that it also calls the formsets' ``save_m2m()``
        methods.
        """
        saved_composites = []
        for name, composite in self.formsets.items():
            field = self.composite_fields[name]
            if hasattr(field, 'save'):
                field.save(self, name, composite, commit=commit)
                saved_composites.append(composite)

        self._extend_save_m2m('save_formsets_m2m', saved_composites)


class SuperModelForm(six.with_metaclass(SuperModelFormMetaclass,
                                        SuperModelFormMixin, forms.ModelForm)):
    pass


class SuperForm(six.with_metaclass(SuperFormMetaclass,
                                   SuperFormMixin, forms.Form)):
    pass



import warnings
warnings.simplefilter('always')

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
    },
}

USE_I18N = True
USE_L10N = True

INSTALLED_APPS = [
    'django_superform',
    'tests',
]

MIDDLEWARE_CLASSES = ()

STATIC_URL = '/static/'

SECRET_KEY = '0'

import django
if django.VERSION < (1, 6):
    TEST_RUNNER = 'discover_runner.DiscoverRunner'



from django.forms.forms import BoundField


class CompositeBoundField(BoundField):
    """
    This is an emulation of the BoundField that is returned if you call
    ``form['field_name']`` for an ordinary django field.

    The ``CompositeBoundField`` has the purpose of providing the same interface
    to other libraries that use a ``BoundField`` instance for rendering.
    """

    # __init__, no changes required
    # __str__, no changes required

    def __iter__(self):
        """
        Iterates over the composite field. For ``FormSetField``s this will
        return form instances. For ``FormField``s this will return bound form
        fields.
        """
        for item in self.form.get_composite_field_value(self.name):
            yield item

    # __len__, no changes required
    # __getitem__, no changes required

    def __nonzero__(self):
        """
        Never evaluate the bound field as False. This is necessary since
        otherwise python will fallback to the __len__ implementation of the
        object. However this might return 0 for a ``CompositeBoundField`` since
        it might contain zero forms in the given formset.
        """
        return True

    @property
    def errors(self):
        """
        Returns an ErrorList for this field, either the errors of the contained
        formset or of the inlined form.
        """
        # TODO: We could make this work and return the forms/formsets errors.
        return self.form.error_class()

    # as_widget, no changes required

    def as_text(self, attrs=None, **kwargs):
        # Not supported. This does not make sense for a CompositeField.
        raise NotImplementedError

    def as_textarea(self, attrs=None, **kwargs):
        # Not supported. This does not make sense for a CompositeField.
        raise NotImplementedError

    def as_hidden(self, attrs=None, **kwargs):
        """
        Returns a string of HTML for representing this as multiple
        <input type="hidden">.
        """
        # TODO: This might make sense. But we don't support it yet.
        raise NotImplementedError

    @property
    def data(self):
        """
        Returns the data for this BoundField, or None if it wasn't given.
        """
        # Not supported. This does not make sense for a CompositeField.
        return None

    def value(self):
        """
        Returns the form/formset for this BoundField. This is passed into the
        call for ``self.widget.render`` in the ``as_widget`` method.
        """
        return self.form.get_composite_field_value(self.name)

    # label_tag, no changes required
    # auto_id, no changes required
    # id_for_label, no changes required



# -*- coding: utf-8 -*-
import codecs
import re
from os import path
from distutils.core import setup
from setuptools import find_packages


def read(*parts):
    return codecs.open(path.join(path.dirname(__file__), *parts),
                       encoding='utf-8').read()


def find_version(*file_paths):
    version_file = read(*file_paths)
    version_match = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]",
                              version_file, re.M)
    if version_match:
        return version_match.group(1)
    raise RuntimeError("Unable to find version string.")


setup(
    name='django-superform',
    version=find_version('django_superform', '__init__.py'),
    author=u'Gregor Müllegger',
    author_email='gregor@muellegger.de',
    packages=find_packages(),
    include_package_data=True,
    url='https://github.com/gregmuellegger/django-superform',
    license='BSD licence, see LICENSE file',
    description='So much easier handling of formsets.',
    long_description=u'\n\n'.join((
        read('README.rst'),
        read('CHANGES.rst'))),
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
    ],
    zip_safe=False,
)



from django import forms
from django.test import TestCase
from django_superform import SuperModelForm, ModelFormField

from .models import Series, Post, Image


class UseFirstModelFormField(ModelFormField):
    def get_instance(self, form, name):
        try:
            return self.form_class._meta.model._default_manager.all()[0]
        except IndexError:
            return None


class SeriesForm(forms.ModelForm):
    class Meta:
        model = Series
        fields = ('title',)


class PostForm(SuperModelForm):
    series = ModelFormField(SeriesForm)

    class Meta:
        model = Post
        fields = ('title',)


class UnrequiredSeriesPostForm(SuperModelForm):
    series = ModelFormField(SeriesForm, required=False)

    class Meta:
        model = Post
        fields = ('title',)


class UseExistingSeriesPostForm(SuperModelForm):
    existing_series = UseFirstModelFormField(SeriesForm)

    class Meta:
        model = Post
        fields = ('title',)


class FormFieldTests(TestCase):
    def test_is_in_composite_fields(self):
        superform = PostForm()
        self.assertTrue('series' in superform.composite_fields)

    def test_is_in_forms_attr(self):
        superform = PostForm()
        self.assertTrue('series' in superform.forms)
        self.assertTrue(isinstance(superform.forms['series'], SeriesForm))

    def test_form_prefix(self):
        superform = PostForm()
        form = superform.forms['series']
        self.assertEqual(form.prefix, 'form-series')

    def test_form_save(self):
        superform = PostForm({
            'title': 'New blog post',
            'form-series-title': 'Unrelated series',
        })
        superform.save()

        self.assertEqual(Series.objects.count(), 1)
        series = Series.objects.get()

        self.assertEqual(series.title, 'Unrelated series')

        self.assertEqual(Post.objects.count(), 1)
        post = Post.objects.get()
        self.assertEqual(post.title, 'New blog post')

        # The relation between post and series IS NOT done by the
        # ModelFormField. It only takes care of nesting. The relation can be
        # handled with the ModelFormField subclasses like ForeignKeyFormField.
        self.assertEqual(post.series, None)

    def test_override_get_instance(self):
        existing_series = Series.objects.create(title='Existing series')

        superform = UseExistingSeriesPostForm()
        self.assertEqual(superform.forms['existing_series'].instance,
                         existing_series)
        self.assertEqual(superform.forms['existing_series']['title'].value(),
                         'Existing series')

    def test_save_existing_instance(self):
        existing_series = Series.objects.create(title='Existing series')

        superform = UseExistingSeriesPostForm({
            'title': 'Blog post',
            'form-existing_series-title': 'Changed name',
        })
        superform.save()

        changed_series = Series.objects.get()
        self.assertEqual(changed_series.pk, existing_series.pk)
        self.assertEqual(changed_series.title, 'Changed name')

    def test_required(self):
        form = PostForm({
            'title': 'New blog post',
            'form-series-title': '',
        })
        form.full_clean()

        self.assertFalse(form.is_valid())
        self.assertTrue('required' in str(form.errors['series']['title']))

        # Test when key is not in data.
        form = PostForm({
            'title': 'New blog post',
        })
        form.full_clean()

        self.assertFalse(form.is_valid())
        self.assertTrue('required' in str(form.errors['series']['title']))

    def test_not_required(self):
        form = UnrequiredSeriesPostForm({
            'title': 'New blog post',
            'form-series-title': '',
        })
        form.full_clean()

        self.assertTrue(form.is_valid())

        form.save()
        self.assertEqual(Post.objects.count(), 1)
        self.assertEqual(Series.objects.count(), 0)



# -*- coding: utf-8 -*-
"""
     ____   _                 _____                 _____
    |    \ |_|___ ___ ___ ___|   __|_ _ ___ ___ ___|   __|___ ___ _____
    |  |  || | .'|   | . | . |__   | | | . | -_|  _|   __| . |  _|     |
    |____/_| |__,|_|_|_  |___|_____|___|  _|___|_| |__|  |___|_| |_|_|_|
         |___|       |___|             |_|


Author: Gregor Müllegger <gregor@muellegger.de>
Project home: https://github.com/gregmuellegger/django-superform
See http://django-superform.readthedocs.org/en/latest/ for complete docs.
"""
from .fields import (
    FormField, ModelFormField, ForeignKeyFormField, FormSetField,
    ModelFormSetField, InlineFormSetField)
from .forms import SuperForm, SuperModelForm
from .widgets import FormWidget, FormSetWidget


__version__ = '0.2.0'


__all__ = (
    'FormField', 'ModelFormField', 'ForeignKeyFormField', 'FormSetField',
    'ModelFormSetField', 'InlineFormSetField', 'SuperForm', 'SuperModelForm',
    'FormWidget', 'FormSetWidget')



from django.db import models


class Series(models.Model):
    """
    Multiple posts can be grouped into a series.
    """

    title = models.CharField(max_length=50)


class Post(models.Model):
    """
    A blog post. I can be part of a series and can contain multiple images.
    """

    title = models.CharField(max_length=50)
    series = models.ForeignKey('Series', null=True, blank=True)


class Image(models.Model):
    post = models.ForeignKey('Post', related_name='images')

    name = models.CharField(max_length=50)
    position = models.PositiveIntegerField(default=0)

    # Is no ImageField to make testing easier.
    image_url = models.URLField()

    class Meta:
        ordering = ('position',)
